-----------------------------------------------------------------------------------------------
-- Client Lua Script for XpTracker
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 
require "Window"
require "GameLib"
 
-----------------------------------------------------------------------------------------------
-- XpTracker Module Definition
-----------------------------------------------------------------------------------------------
local XpTracker = {}
local sXpStr = "XP"
local sLvlStr = "next level"
local iXpPrevTick = 0
local iXpTotal = 0
local iSecondsRunning = 0
 
-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------
-- e.g. local kiExampleVariableMax = 999
 
-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function XpTracker:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 
    return o
end

function XpTracker:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end

-----------------------------------------------------------------------------------------------
-- XpTracker OnLoad
-----------------------------------------------------------------------------------------------
function XpTracker:OnLoad()
	Apollo.RegisterSlashCommand("exp", "OnXpTrackerOn", self)
	Apollo.RegisterEventHandler("ExperienceGained", "OnExperienceGain", self)
	Apollo.RegisterEventHandler("ElderPointsGained", "OnElderPointGain", self)
	self.timer = ApolloTimer.Create(1.0, true, "OnTimer", self)
	
	-- if we're level 50, set the info to match
	if self:IsLevel50() then
		self:SetTextEP()
	end
end

-----------------------------------------------------------------------------------------------
-- XpTracker Functions
-----------------------------------------------------------------------------------------------
function XpTracker:SetTextEP()
	if sXpStr ~= "Elder Gems" then
		sXpStr = "Elder Gems"
		sLvlStr = "cap Elder Gems"
		iXpTotal = 0
	end
end

-- on SlashCommand "/exp"
function XpTracker:OnXpTrackerOn(cmd, args)
	if args == "reset" then
		self:WriteLog("Reset " .. sXpStr .. " stats")
		iXpTotal = 0
		iSecondsRunning = 0
	else
		self:PrintXp()
	end
end

function XpTracker:OnExperienceGain(eReason, sTargetID, sText, nDelay)
	local sXp = sText:gsub("%D", "")
	local iXp = tonumber(sXp)
	iXpTotal = iXpTotal + iXp
end

function XpTracker:OnElderPointGain(nAmount)
	self:SetTextEP()
	iXpTotal = iXpTotal + nAmount
end

function XpTracker:OnTimer()

	-- check if we're 50 and have capped EP for the week. if so, stop reporting.
	if self:IsLevel50() and self:GetNeededXP() <= 0 then
		self:WriteLog("Elder Points are capped for the week. Turning off xpTracker automatic reporting for this session.")
		self.timer = nil
	else
		-- update the seconds running
		iSecondsRunning = iSecondsRunning + 1
		
		-- report progress every 5 mins
		if iSecondsRunning % 300 == 0 then
			self:PrintXp()
		end
	end
	
end

function XpTracker:PrintXp()
	if iXpTotal == 0 or iSecondsRunning == 0 then
		self:WriteLog("No " .. sXpStr .. " gained this session")
	else
		-- get stats
		local iXpPerSecond = iXpTotal / iSecondsRunning
		
		-- get how much XP we need to level
		local nCurrentXP = self:GetCurrentXP()
		local nNeededXP = self:GetNeededXP()
		
		-- get how long it would take to level
		local nSecondsToLevel = nNeededXP / iXpPerSecond
		
		-- pretty up the output
		local sPrettyTotal = self:CommaValue(iXpTotal)
		local sPrettyPerHour  = self:CommaValue(self:Round(iXpPerSecond * 3600))
		local sPrettyTTL = self:PrettifySeconds(nSecondsToLevel)
		
		-- print the output
		self:WriteLog("Total " .. sXpStr .. " in session: " .. sPrettyTotal)
		self:WriteLog(sXpStr .. " per hour: " .. sPrettyPerHour)
		self:WriteLog("Time to " .. sLvlStr .. ": " .. sPrettyTTL)
	end

end

function XpTracker:IsLevel50()
	local unitPlayer = GameLib.GetPlayerUnit()
	if unitPlayer == nil then
		return false
	end
	
	local tStats = unitPlayer:GetBasicStats()
	if tStats.nLevel == 50 then
		return true
	else
		return false
	end
end

function XpTracker:GetCurrentXP()
	if self:IsLevel50() then
		return GetElderPoints()
	else
		local nCurrentXP = GetXp() - GetXpToCurrentLevel()
		return nCurrentXP
	end
end

function XpTracker:GetNeededXP()
	if self:IsLevel50() then
		--local nEPToAGem = GameLib.ElderPointsPerGem
		local nEPMax = GameLib.ElderPointsDailyMax
		local nCurrentEP = self:GetCurrentXP()
		return nEPMax - nCurrentEP
	else
		local nXpForLevel = GetXpToNextLevel()
		local nCurrentXP = self:GetCurrentXP()
		return nXpForLevel - nCurrentXP
	end
end

function XpTracker:WriteLog(strText)
	ChatSystemLib.PostOnChannel(ChatSystemLib.ChatChannel_System, strText, "xpTracker")
end

function XpTracker:PrettifySeconds(seconds)
	local minute = 60
	local hour = minute * 60
	local day = hour * 24
	local week = day * 7
	
	if seconds < minute then
		return "Less than a minute!"
	elseif seconds < hour then
		local nMinutes = self:Round(seconds / 60, 1)
		if nMinutes == 1 then
			return "About a minute"
		else
			return "About " .. nMinutes .. " minutes"
		end
	elseif seconds < week then
		local nHours = self:Round(seconds / hour)
		if nHours == 1 then
			return "About an hour"
		else
			return "About " .. nHours .. " hours"
		end
	else
		return "Longer than a week :("
	end
end

-- from http://lua-users.org/wiki/FormattingNumbers
function XpTracker:CommaValue(amount)
	local formatted = amount
	
	if self:IsLevel50() then
		formatted = self:Round(formatted / GameLib.ElderPointsPerGem, 2)
	end
	
	while true do  
		formatted, k = string.gsub(formatted, "^(-?%d+)(%d%d%d)", '%1,%2')
		if (k==0) then
			break
		end
	end
	return formatted
end

-- from http://lua-users.org/wiki/FormattingNumbers
function XpTracker:Round(val, decimal)
	if (decimal) then
		return math.floor( (val * 10^decimal) + 0.5) / (10^decimal)
	else
		return math.floor(val+0.5)
	end
end

-----------------------------------------------------------------------------------------------
-- XpTracker Instance
-----------------------------------------------------------------------------------------------
local XpTrackerInst = XpTracker:new()
XpTrackerInst:Init()
